@@ -1,5 +1,4 @@
 use ibc_proto::ibc::core::commitment::v1::MerkleProof;
-use tendermint::Time;
 
 use crate::clients::ics07_tendermint::client_def::TendermintClient;
 use crate::core::ics02_client::client_consensus::{AnyConsensusState, ConsensusState};
@@ -11,6 +10,7 @@ use crate::core::ics02_client::header::{AnyHeader, Header};
 use crate::core::ics03_connection::connection::ConnectionEnd;
 use crate::core::ics04_channel::channel::ChannelEnd;
 use crate::core::ics04_channel::context::ChannelReader;
+use crate::core::ics04_channel::msgs::acknowledgement::Acknowledgement;
 use crate::core::ics04_channel::packet::Sequence;
 use crate::core::ics23_commitment::commitment::{
     CommitmentPrefix, CommitmentProofBytes, CommitmentRoot,
@@ -30,7 +30,6 @@ pub trait ClientDef: Clone {
 
     fn check_header_and_update_state(
         &self,
-        now: Time,
         ctx: &dyn ClientReader,
         client_id: ClientId,
         client_state: Self::ClientState,
@@ -47,7 +46,7 @@ pub trait ClientDef: Clone {
     ) -> Result<(Self::ClientState, Self::ConsensusState), Error>;
 
     /// Verification functions as specified in:
-    /// <https://github.com/cosmos/ibc/tree/master/spec/ics-002-client-semantics>
+    /// <https://github.com/cosmos/ibc/tree/master/spec/core/ics-002-client-semantics>
     ///
     /// Verify a `proof` that the consensus state of a given client (at height `consensus_height`)
     /// matches the input `consensus_state`. The parameter `counterparty_height` represent the
@@ -135,7 +134,7 @@ pub trait ClientDef: Clone {
         port_id: &PortId,
         channel_id: &ChannelId,
         sequence: Sequence,
-        ack: Vec<u8>,
+        ack: Acknowledgement,
     ) -> Result<(), Error>;
 
     /// Verify a `proof` that of the next_seq_received.
@@ -197,7 +196,6 @@ impl ClientDef for AnyClient {
     /// Validates an incoming `header` against the latest consensus state of this client.
     fn check_header_and_update_state(
         &self,
-        now: Time,
         ctx: &dyn ClientReader,
         client_id: ClientId,
         client_state: AnyClientState,
@@ -211,13 +209,8 @@ impl ClientDef for AnyClient {
                 )
                 .ok_or_else(|| Error::client_args_type_mismatch(ClientType::Tendermint))?;
 
-                let (new_state, new_consensus) = client.check_header_and_update_state(
-                    now,
-                    ctx,
-                    client_id,
-                    client_state,
-                    header,
-                )?;
+                let (new_state, new_consensus) =
+                    client.check_header_and_update_state(ctx, client_id, client_state, header)?;
 
                 Ok((
                     AnyClientState::Tendermint(new_state),
@@ -233,13 +226,8 @@ impl ClientDef for AnyClient {
                 )
                 .ok_or_else(|| Error::client_args_type_mismatch(ClientType::Mock))?;
 
-                let (new_state, new_consensus) = client.check_header_and_update_state(
-                    now,
-                    ctx,
-                    client_id,
-                    client_state,
-                    header,
-                )?;
+                let (new_state, new_consensus) =
+                    client.check_header_and_update_state(ctx, client_id, client_state, header)?;
 
                 Ok((
                     AnyClientState::Mock(new_state),
@@ -506,7 +494,7 @@ impl ClientDef for AnyClient {
         port_id: &PortId,
         channel_id: &ChannelId,
         sequence: Sequence,
-        ack: Vec<u8>,
+        ack: Acknowledgement,
     ) -> Result<(), Error> {
         match self {
             Self::Tendermint(client) => {
