@@ -8,6 +8,7 @@ use crate::core::ics02_client::client_type::ClientType;
 use crate::core::ics02_client::error::{Error, ErrorDetail};
 use crate::core::ics02_client::handler::ClientResult::{self, Create, Update, Upgrade};
 use crate::core::ics24_host::identifier::ClientId;
+use crate::timestamp::Timestamp;
 use crate::Height;
 
 /// Defines the read-only part of ICS2 (client functions) context.
@@ -58,6 +59,20 @@ pub trait ClientReader {
     /// Returns the current height of the local chain.
     fn host_height(&self) -> Height;
 
+    /// Returns the current timestamp of the local chain.
+    fn host_timestamp(&self) -> Timestamp {
+        let pending_consensus_state = self
+            .pending_host_consensus_state()
+            .expect("host must have pending consensus state");
+        pending_consensus_state.timestamp()
+    }
+
+    /// Returns the `ConsensusState` of the host (local) chain at a specific height.
+    fn host_consensus_state(&self, height: Height) -> Result<AnyConsensusState, Error>;
+
+    /// Returns the pending `ConsensusState` of the host (local) chain.
+    fn pending_host_consensus_state(&self) -> Result<AnyConsensusState, Error>;
+
     /// Returns a natural number, counting how many clients have been created thus far.
     /// The value of this counter should increase only via method `ClientKeeper::increase_client_counter`.
     fn client_counter(&self) -> Result<u64, Error>;
@@ -78,8 +93,16 @@ pub trait ClientKeeper {
                     res.consensus_state,
                 )?;
                 self.increase_client_counter();
-                self.store_update_time(res.client_id.clone(), res.client_state.latest_height())?;
-                self.store_update_height(res.client_id, res.client_state.latest_height())?;
+                self.store_update_time(
+                    res.client_id.clone(),
+                    res.client_state.latest_height(),
+                    res.processed_time,
+                )?;
+                self.store_update_height(
+                    res.client_id,
+                    res.client_state.latest_height(),
+                    res.processed_height,
+                )?;
                 Ok(())
             }
             Update(res) => {
@@ -89,8 +112,16 @@ pub trait ClientKeeper {
                     res.client_state.latest_height(),
                     res.consensus_state,
                 )?;
-                self.store_update_time(res.client_id.clone(), res.client_state.latest_height())?;
-                self.store_update_height(res.client_id, res.client_state.latest_height())?;
+                self.store_update_time(
+                    res.client_id.clone(),
+                    res.client_state.latest_height(),
+                    res.processed_time,
+                )?;
+                self.store_update_height(
+                    res.client_id,
+                    res.client_state.latest_height(),
+                    res.processed_height,
+                )?;
                 Ok(())
             }
             Upgrade(res) => {
@@ -133,12 +164,22 @@ pub trait ClientKeeper {
     fn increase_client_counter(&mut self);
 
     /// Called upon successful client update.
-    /// Implementations are expected to use this to record the current (host) time as the time at
-    /// which this update (or header) was processed.
-    fn store_update_time(&mut self, client_id: ClientId, height: Height) -> Result<(), Error>;
+    /// Implementations are expected to use this to record the specified time as the time at which
+    /// this update (or header) was processed.
+    fn store_update_time(
+        &mut self,
+        client_id: ClientId,
+        height: Height,
+        timestamp: Timestamp,
+    ) -> Result<(), Error>;
 
     /// Called upon successful client update.
-    /// Implementations are expected to use this to record the current (host) height as the height
+    /// Implementations are expected to use this to record the specified height as the height at
     /// at which this update (or header) was processed.
-    fn store_update_height(&mut self, client_id: ClientId, height: Height) -> Result<(), Error>;
+    fn store_update_height(
+        &mut self,
+        client_id: ClientId,
+        height: Height,
+        host_height: Height,
+    ) -> Result<(), Error>;
 }
