@@ -249,6 +249,25 @@ define_error! {
             [ Ics07Error ]
             | _ | { format_args!("Tendermint-specific handler error") },
 
+        MissingLocalConsensusState
+            { height: Height }
+            | e | { format_args!("the local consensus state could not be retrieved for height {}", e.height) },
+
+        InvalidConnectionEnd
+            [ TraceError<TendermintProtoError>]
+            | _ | { "invalid connection end" },
+
+        InvalidChannelEnd
+            [ TraceError<TendermintProtoError>]
+            | _ | { "invalid channel end" },
+
+        InvalidAnyClientState
+            [ TraceError<TendermintProtoError>]
+            | _ | { "invalid any client state" },
+
+        InvalidAnyConsensusState
+            [ TraceError<TendermintProtoError> ]
+            | _ | { "invalid any client consensus state" },
     }
 }
 
