@@ -1,7 +1,5 @@
 //! Protocol logic specific to processing ICS2 messages of type `MsgUpdateAnyClient`.
 
-use core::convert::From;
-use tendermint::Time;
 use tracing::debug;
 
 use crate::core::ics02_client::client_consensus::AnyConsensusState;
@@ -12,6 +10,7 @@ use crate::core::ics02_client::error::Error;
 use crate::core::ics02_client::events::Attributes;
 use crate::core::ics02_client::handler::ClientResult;
 use crate::core::ics02_client::header::Header;
+use crate::core::ics02_client::height::Height;
 use crate::core::ics02_client::msgs::update_client::MsgUpdateAnyClient;
 use crate::core::ics24_host::identifier::ClientId;
 use crate::events::IbcEvent;
@@ -26,10 +25,11 @@ pub struct Result {
     pub client_id: ClientId,
     pub client_state: AnyClientState,
     pub consensus_state: AnyConsensusState,
+    pub processed_time: Timestamp,
+    pub processed_height: Height,
 }
 
 pub fn process(
-    now: Time,
     ctx: &dyn ClientReader,
     msg: MsgUpdateAnyClient,
 ) -> HandlerResult<ClientResult, Error> {
@@ -62,13 +62,11 @@ pub fn process(
 
     debug!("latest consensus state: {:?}", latest_consensus_state);
 
-    let duration = Timestamp::from(now)
+    let now = ctx.host_timestamp();
+    let duration = now
         .duration_since(&latest_consensus_state.timestamp())
         .ok_or_else(|| {
-            Error::invalid_consensus_state_timestamp(
-                latest_consensus_state.timestamp(),
-                Timestamp::from(now),
-            )
+            Error::invalid_consensus_state_timestamp(latest_consensus_state.timestamp(), now)
         })?;
 
     if client_state.expired(duration) {
@@ -82,17 +80,20 @@ pub fn process(
     // This function will return the new client_state (its latest_height changed) and a
     // consensus_state obtained from header. These will be later persisted by the keeper.
     let (new_client_state, new_consensus_state) = client_def
-        .check_header_and_update_state(now, ctx, client_id.clone(), client_state, header)
+        .check_header_and_update_state(ctx, client_id.clone(), client_state, header)
         .map_err(|e| Error::header_verification_failure(e.to_string()))?;
 
     let result = ClientResult::Update(Result {
         client_id: client_id.clone(),
         client_state: new_client_state,
         consensus_state: new_consensus_state,
+        processed_time: ctx.host_timestamp(),
+        processed_height: ctx.host_height(),
     });
 
     let event_attributes = Attributes {
         client_id,
+        height: ctx.host_height(),
         ..Default::default()
     };
     output.emit(IbcEvent::UpdateClient(event_attributes.into()));
@@ -103,12 +104,12 @@ pub fn process(
 #[cfg(test)]
 mod tests {
     use core::str::FromStr;
-    use tendermint::Time;
     use test_log::test;
 
     use crate::core::ics02_client::client_consensus::AnyConsensusState;
     use crate::core::ics02_client::client_state::{AnyClientState, ClientState};
     use crate::core::ics02_client::client_type::ClientType;
+    use crate::core::ics02_client::context::ClientReader;
     use crate::core::ics02_client::error::{Error, ErrorDetail};
     use crate::core::ics02_client::handler::dispatch;
     use crate::core::ics02_client::handler::ClientResult::Update;
@@ -143,7 +144,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
         match output {
             Ok(HandlerOutput {
@@ -154,8 +155,9 @@ mod tests {
                 assert_eq!(events.len(), 1);
                 let event = events.pop().unwrap();
                 assert!(
-                    matches!(event, IbcEvent::UpdateClient(e) if e.client_id() == &msg.client_id)
+                    matches!(event, IbcEvent::UpdateClient(ref e) if e.client_id() == &msg.client_id)
                 );
+                assert_eq!(event.height(), ctx.host_height());
                 assert!(log.is_empty());
                 // Check the result
                 match result {
@@ -190,7 +192,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
         match output {
             Err(Error(ErrorDetail::ClientNotFound(e), _)) => {
@@ -226,7 +228,7 @@ mod tests {
                 signer: signer.clone(),
             };
 
-            let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+            let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
             match output {
                 Ok(HandlerOutput {
@@ -237,8 +239,9 @@ mod tests {
                     assert_eq!(events.len(), 1);
                     let event = events.pop().unwrap();
                     assert!(
-                        matches!(event, IbcEvent::UpdateClient(e) if e.client_id() == &msg.client_id)
+                        matches!(event, IbcEvent::UpdateClient(ref e) if e.client_id() == &msg.client_id)
                     );
+                    assert_eq!(event.height(), ctx.host_height());
                     assert!(log.is_empty());
                 }
                 Err(err) => {
@@ -293,7 +296,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
         match output {
             Ok(HandlerOutput {
@@ -304,8 +307,9 @@ mod tests {
                 assert_eq!(events.len(), 1);
                 let event = events.pop().unwrap();
                 assert!(
-                    matches!(event, IbcEvent::UpdateClient(e) if e.client_id() == &msg.client_id)
+                    matches!(event, IbcEvent::UpdateClient(ref e) if e.client_id() == &msg.client_id)
                 );
+                assert_eq!(event.height(), ctx.host_height());
                 assert!(log.is_empty());
                 // Check the result
                 match result {
@@ -370,7 +374,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
         match output {
             Ok(HandlerOutput {
@@ -381,8 +385,9 @@ mod tests {
                 assert_eq!(events.len(), 1);
                 let event = events.pop().unwrap();
                 assert!(
-                    matches!(event, IbcEvent::UpdateClient(e) if e.client_id() == &msg.client_id)
+                    matches!(event, IbcEvent::UpdateClient(ref e) if e.client_id() == &msg.client_id)
                 );
+                assert_eq!(event.height(), ctx.host_height());
                 assert!(log.is_empty());
                 // Check the result
                 match result {
@@ -450,7 +455,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg.clone()));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg.clone()));
 
         match output {
             Ok(HandlerOutput {
@@ -461,8 +466,9 @@ mod tests {
                 assert_eq!(events.len(), 1);
                 let event = events.pop().unwrap();
                 assert!(
-                    matches!(event, IbcEvent::UpdateClient(e) if e.client_id() == &msg.client_id)
+                    matches!(event, IbcEvent::UpdateClient(ref e) if e.client_id() == &msg.client_id)
                 );
+                assert_eq!(event.height(), ctx.host_height());
                 assert!(log.is_empty());
                 // Check the result
                 match result {
@@ -524,7 +530,7 @@ mod tests {
             signer,
         };
 
-        let output = dispatch(Time::now(), &ctx, ClientMsg::UpdateClient(msg));
+        let output = dispatch(&ctx, ClientMsg::UpdateClient(msg));
 
         match output {
             Ok(_) => {
