@@ -21,23 +21,11 @@ pub const TYPE_URL: &str = "/ibc.core.connection.v1.MsgConnectionOpenInit";
 pub struct MsgConnectionOpenInit {
     pub client_id: ClientId,
     pub counterparty: Counterparty,
-    pub version: Version,
+    pub version: Option<Version>,
     pub delay_period: Duration,
     pub signer: Signer,
 }
 
-impl MsgConnectionOpenInit {
-    /// Getter: borrow the `client_id` from this message.
-    pub fn client_id(&self) -> &ClientId {
-        &self.client_id
-    }
-
-    /// Getter: borrow the `counterparty` from this message.
-    pub fn counterparty(&self) -> &Counterparty {
-        &self.counterparty
-    }
-}
-
 impl Msg for MsgConnectionOpenInit {
     type ValidationError = Error;
     type Raw = RawMsgConnectionOpenInit;
@@ -63,7 +51,7 @@ impl TryFrom<RawMsgConnectionOpenInit> for MsgConnectionOpenInit {
                 .counterparty
                 .ok_or_else(Error::missing_counterparty)?
                 .try_into()?,
-            version: msg.version.ok_or_else(Error::empty_versions)?.try_into()?,
+            version: msg.version.map(|version| version.try_into()).transpose()?,
             delay_period: Duration::from_nanos(msg.delay_period),
             signer: msg.signer.into(),
         })
@@ -75,7 +63,7 @@ impl From<MsgConnectionOpenInit> for RawMsgConnectionOpenInit {
         RawMsgConnectionOpenInit {
             client_id: ics_msg.client_id.as_str().to_string(),
             counterparty: Some(ics_msg.counterparty.into()),
-            version: Some(ics_msg.version.into()),
+            version: ics_msg.version.map(|version| version.into()),
             delay_period: ics_msg.delay_period.as_nanos() as u64,
             signer: ics_msg.signer.to_string(),
         }
