@@ -36,29 +36,35 @@ const PKT_ACK_ATTRIBUTE_KEY: &str = "packet_ack";
 pub fn try_from_tx(event: &tendermint::abci::Event) -> Option<IbcEvent> {
     match event.type_str.parse() {
         Ok(IbcEventType::OpenInitChannel) => extract_attributes_from_tx(event)
-            .map(OpenInit::from)
-            .map(IbcEvent::OpenInitChannel)
-            .ok(),
+            .map(OpenInit::try_from)
+            .map(|res| res.ok().map(IbcEvent::OpenInitChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::OpenTryChannel) => extract_attributes_from_tx(event)
-            .map(OpenTry::from)
-            .map(IbcEvent::OpenTryChannel)
-            .ok(),
+            .map(OpenTry::try_from)
+            .map(|res| res.ok().map(IbcEvent::OpenTryChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::OpenAckChannel) => extract_attributes_from_tx(event)
-            .map(OpenAck::from)
-            .map(IbcEvent::OpenAckChannel)
-            .ok(),
+            .map(OpenAck::try_from)
+            .map(|res| res.ok().map(IbcEvent::OpenAckChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::OpenConfirmChannel) => extract_attributes_from_tx(event)
-            .map(OpenConfirm::from)
-            .map(IbcEvent::OpenConfirmChannel)
-            .ok(),
+            .map(OpenConfirm::try_from)
+            .map(|res| res.ok().map(IbcEvent::OpenConfirmChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::CloseInitChannel) => extract_attributes_from_tx(event)
-            .map(CloseInit::from)
-            .map(IbcEvent::CloseInitChannel)
-            .ok(),
+            .map(CloseInit::try_from)
+            .map(|res| res.ok().map(IbcEvent::CloseInitChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::CloseConfirmChannel) => extract_attributes_from_tx(event)
-            .map(CloseConfirm::from)
-            .map(IbcEvent::CloseConfirmChannel)
-            .ok(),
+            .map(CloseConfirm::try_from)
+            .map(|res| res.ok().map(IbcEvent::CloseConfirmChannel))
+            .ok()
+            .flatten(),
         Ok(IbcEventType::SendPacket) => {
             extract_packet_and_write_ack_from_tx(event)
                 .map(|(packet, write_ack)| {
@@ -250,7 +256,7 @@ impl From<Attributes> for Vec<Tag> {
         if let Some(channel_id) = a.channel_id {
             let channel_id = Tag {
                 key: CHANNEL_ID_ATTRIBUTE_KEY.parse().unwrap(),
-                value: channel_id.as_str().parse().unwrap(),
+                value: channel_id.to_string().parse().unwrap(),
             };
             attributes.push(channel_id);
         }
@@ -267,7 +273,7 @@ impl From<Attributes> for Vec<Tag> {
         if let Some(channel_id) = a.counterparty_channel_id {
             let channel_id = Tag {
                 key: COUNTERPARTY_CHANNEL_ID_ATTRIBUTE_KEY.parse().unwrap(),
-                value: channel_id.as_str().parse().unwrap(),
+                value: channel_id.to_string().parse().unwrap(),
             };
             attributes.push(channel_id);
         }
@@ -348,29 +354,40 @@ trait EventType {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct OpenInit(pub Attributes);
+pub struct OpenInit {
+    pub height: Height,
+    pub port_id: PortId,
+    pub channel_id: Option<ChannelId>,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+    pub counterparty_channel_id: Option<ChannelId>,
+}
 
 impl OpenInit {
-    pub fn attributes(&self) -> &Attributes {
-        &self.0
-    }
     pub fn channel_id(&self) -> Option<&ChannelId> {
-        self.0.channel_id.as_ref()
+        self.channel_id.as_ref()
     }
     pub fn port_id(&self) -> &PortId {
-        &self.0.port_id
+        &self.port_id
     }
     pub fn height(&self) -> Height {
-        self.0.height
+        self.height
     }
     pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
+        self.height = height;
     }
 }
 
-impl From<Attributes> for OpenInit {
-    fn from(attrs: Attributes) -> Self {
-        OpenInit(attrs)
+impl From<OpenInit> for Attributes {
+    fn from(ev: OpenInit) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: ev.channel_id,
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
     }
 }
 
@@ -387,29 +404,39 @@ impl EventType for OpenInit {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct OpenTry(pub Attributes);
-
-impl OpenTry {
-    pub fn attributes(&self) -> &Attributes {
-        &self.0
-    }
-    pub fn channel_id(&self) -> Option<&ChannelId> {
-        self.0.channel_id.as_ref()
-    }
-    pub fn port_id(&self) -> &PortId {
-        &self.0.port_id
-    }
-    pub fn height(&self) -> Height {
-        self.0.height
-    }
-    pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
-    }
+pub struct OpenTry {
+    pub height: Height,
+    pub port_id: PortId,
+    pub channel_id: Option<ChannelId>,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+    pub counterparty_channel_id: Option<ChannelId>,
 }
 
-impl From<Attributes> for OpenTry {
-    fn from(attrs: Attributes) -> Self {
-        OpenTry(attrs)
+impl From<OpenTry> for Attributes {
+    fn from(ev: OpenTry) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: ev.channel_id,
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
+    }
+}
+impl OpenTry {
+    pub fn channel_id(&self) -> Option<&ChannelId> {
+        self.channel_id.as_ref()
+    }
+    pub fn port_id(&self) -> &PortId {
+        &self.port_id
+    }
+    pub fn height(&self) -> Height {
+        self.height
+    }
+    pub fn set_height(&mut self, height: Height) {
+        self.height = height;
     }
 }
 
@@ -426,33 +453,44 @@ impl EventType for OpenTry {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct OpenAck(pub Attributes);
+pub struct OpenAck {
+    pub height: Height,
+    pub port_id: PortId,
+    pub channel_id: Option<ChannelId>,
+    pub counterparty_channel_id: Option<ChannelId>,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+}
 
-impl OpenAck {
-    pub fn attributes(&self) -> &Attributes {
-        &self.0
-    }
-    pub fn channel_id(&self) -> Option<&ChannelId> {
-        self.0.channel_id.as_ref()
-    }
-    pub fn port_id(&self) -> &PortId {
-        &self.0.port_id
-    }
-    pub fn height(&self) -> Height {
-        self.0.height
-    }
-    pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
-    }
-
-    pub fn counterparty_channel_id(&self) -> Option<&ChannelId> {
-        self.0.counterparty_channel_id.as_ref()
+impl From<OpenAck> for Attributes {
+    fn from(ev: OpenAck) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: ev.channel_id,
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
     }
 }
 
-impl From<Attributes> for OpenAck {
-    fn from(attrs: Attributes) -> Self {
-        OpenAck(attrs)
+impl OpenAck {
+    pub fn channel_id(&self) -> Option<&ChannelId> {
+        self.channel_id.as_ref()
+    }
+    pub fn port_id(&self) -> &PortId {
+        &self.port_id
+    }
+    pub fn height(&self) -> Height {
+        self.height
+    }
+    pub fn set_height(&mut self, height: Height) {
+        self.height = height;
+    }
+
+    pub fn counterparty_channel_id(&self) -> Option<&ChannelId> {
+        self.counterparty_channel_id.as_ref()
     }
 }
 
@@ -469,29 +507,40 @@ impl EventType for OpenAck {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct OpenConfirm(pub Attributes);
+pub struct OpenConfirm {
+    pub height: Height,
+    pub port_id: PortId,
+    pub channel_id: Option<ChannelId>,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+    pub counterparty_channel_id: Option<ChannelId>,
+}
 
-impl OpenConfirm {
-    pub fn attributes(&self) -> &Attributes {
-        &self.0
-    }
-    pub fn channel_id(&self) -> Option<&ChannelId> {
-        self.0.channel_id.as_ref()
-    }
-    pub fn port_id(&self) -> &PortId {
-        &self.0.port_id
-    }
-    pub fn height(&self) -> Height {
-        self.0.height
-    }
-    pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
+impl From<OpenConfirm> for Attributes {
+    fn from(ev: OpenConfirm) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: ev.channel_id,
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
     }
 }
 
-impl From<Attributes> for OpenConfirm {
-    fn from(attrs: Attributes) -> Self {
-        OpenConfirm(attrs)
+impl OpenConfirm {
+    pub fn channel_id(&self) -> Option<&ChannelId> {
+        self.channel_id.as_ref()
+    }
+    pub fn port_id(&self) -> &PortId {
+        &self.port_id
+    }
+    pub fn height(&self) -> Height {
+        self.height
+    }
+    pub fn set_height(&mut self, height: Height) {
+        self.height = height;
     }
 }
 
@@ -508,41 +557,69 @@ impl EventType for OpenConfirm {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct CloseInit(pub Attributes);
+pub struct CloseInit {
+    pub height: Height,
+    pub port_id: PortId,
+    pub channel_id: ChannelId,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+    pub counterparty_channel_id: Option<ChannelId>,
+}
 
-impl CloseInit {
-    pub fn port_id(&self) -> &PortId {
-        &self.0.port_id
-    }
-
-    pub fn channel_id(&self) -> &ChannelId {
-        // FIXME(romac): Rework encoding of IbcEvents which use `Attributes`
-        self.0
-            .channel_id
-            .as_ref()
-            .expect("CloseInit should always have a channel_id")
-    }
-
-    pub fn counterparty_port_id(&self) -> &PortId {
-        &self.0.counterparty_port_id
-    }
-
-    pub fn counterparty_channel_id(&self) -> Option<&ChannelId> {
-        self.0.counterparty_channel_id.as_ref()
-    }
-
-    pub fn height(&self) -> Height {
-        self.0.height
-    }
-
-    pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
+impl From<CloseInit> for Attributes {
+    fn from(ev: CloseInit) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: Some(ev.channel_id),
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
     }
 }
 
-impl From<Attributes> for CloseInit {
-    fn from(attrs: Attributes) -> Self {
-        CloseInit(attrs)
+impl CloseInit {
+    pub fn port_id(&self) -> &PortId {
+        &self.port_id
+    }
+
+    pub fn channel_id(&self) -> &ChannelId {
+        &self.channel_id
+    }
+
+    pub fn counterparty_port_id(&self) -> &PortId {
+        &self.counterparty_port_id
+    }
+
+    pub fn counterparty_channel_id(&self) -> Option<&ChannelId> {
+        self.counterparty_channel_id.as_ref()
+    }
+
+    pub fn height(&self) -> Height {
+        self.height
+    }
+
+    pub fn set_height(&mut self, height: Height) {
+        self.height = height;
+    }
+}
+
+impl TryFrom<Attributes> for CloseInit {
+    type Error = EventError;
+    fn try_from(attrs: Attributes) -> Result<Self, Self::Error> {
+        if let Some(channel_id) = attrs.channel_id() {
+            Ok(CloseInit {
+                height: attrs.height,
+                port_id: attrs.port_id.clone(),
+                channel_id: *channel_id,
+                connection_id: attrs.connection_id.clone(),
+                counterparty_port_id: attrs.counterparty_port_id.clone(),
+                counterparty_channel_id: attrs.counterparty_channel_id,
+            })
+        } else {
+            Err(EventError::channel(Error::missing_channel_id()))
+        }
     }
 }
 
@@ -559,7 +636,7 @@ impl core::fmt::Display for CloseInit {
             "{} {} {:?}",
             self.height(),
             IbcEventType::CloseInitChannel.as_str(),
-            self.0
+            Attributes::from(self.clone())
         )
     }
 }
@@ -571,23 +648,37 @@ impl EventType for CloseInit {
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone, PartialEq, Eq)]
-pub struct CloseConfirm(pub Attributes);
+pub struct CloseConfirm {
+    pub height: Height,
+    pub channel_id: Option<ChannelId>,
+    pub port_id: PortId,
+    pub connection_id: ConnectionId,
+    pub counterparty_port_id: PortId,
+    pub counterparty_channel_id: Option<ChannelId>,
+}
 
-impl CloseConfirm {
-    pub fn channel_id(&self) -> Option<&ChannelId> {
-        self.0.channel_id.as_ref()
-    }
-    pub fn height(&self) -> Height {
-        self.0.height
-    }
-    pub fn set_height(&mut self, height: Height) {
-        self.0.height = height;
+impl From<CloseConfirm> for Attributes {
+    fn from(ev: CloseConfirm) -> Self {
+        Self {
+            height: ev.height,
+            port_id: ev.port_id,
+            channel_id: ev.channel_id,
+            connection_id: ev.connection_id,
+            counterparty_port_id: ev.counterparty_port_id,
+            counterparty_channel_id: ev.counterparty_channel_id,
+        }
     }
 }
 
-impl From<Attributes> for CloseConfirm {
-    fn from(attrs: Attributes) -> Self {
-        CloseConfirm(attrs)
+impl CloseConfirm {
+    pub fn channel_id(&self) -> Option<&ChannelId> {
+        self.channel_id.as_ref()
+    }
+    pub fn height(&self) -> Height {
+        self.height
+    }
+    pub fn set_height(&mut self, height: Height) {
+        self.height = height;
     }
 }
 
@@ -603,11 +694,32 @@ impl EventType for CloseConfirm {
     }
 }
 
+macro_rules! impl_try_from_attribute_for_event {
+    ($($event:ty),+) => {
+        $(impl TryFrom<Attributes> for $event {
+            type Error = EventError;
+
+            fn try_from(attrs: Attributes) -> Result<Self, Self::Error> {
+                Ok(Self {
+                    height: attrs.height,
+                    port_id: attrs.port_id,
+                    channel_id: attrs.channel_id,
+                    connection_id: attrs.connection_id,
+                    counterparty_port_id: attrs.counterparty_port_id,
+                    counterparty_channel_id: attrs.counterparty_channel_id,
+                })
+            }
+        })+
+    };
+}
+
+impl_try_from_attribute_for_event!(OpenInit, OpenTry, OpenAck, OpenConfirm, CloseConfirm);
+
 macro_rules! impl_from_ibc_to_abci_event {
     ($($event:ty),+) => {
         $(impl From<$event> for AbciEvent {
             fn from(v: $event) -> Self {
-                let attributes = Vec::<Tag>::from(v.0);
+                let attributes = Vec::<Tag>::from(Attributes::from(v));
                 let type_str = <$event>::event_type().as_str().to_string();
                 AbciEvent {
                     type_str,
@@ -633,7 +745,7 @@ macro_rules! impl_try_from_raw_obj_for_event {
             type Error = EventError;
 
             fn try_from(obj: RawObject<'_>) -> Result<Self, Self::Error> {
-                Ok(Self(extract_attributes(&obj, Self::event_type().as_str())?))
+                extract_attributes(&obj, Self::event_type().as_str())?.try_into()
             }
         })+
     };
@@ -1000,34 +1112,46 @@ mod tests {
         let attributes = Attributes {
             height: Height::default(),
             port_id: "test_port".parse().unwrap(),
-            channel_id: Some("test_channel".parse().unwrap()),
+            channel_id: Some("channel-0".parse().unwrap()),
             connection_id: "test_connection".parse().unwrap(),
             counterparty_port_id: "counterparty_test_port".parse().unwrap(),
-            counterparty_channel_id: Some("counterparty_test_channel".parse().unwrap()),
+            counterparty_channel_id: Some("channel-1".parse().unwrap()),
         };
         let mut abci_events = vec![];
-        let open_init = OpenInit::from(attributes.clone());
+        let open_init = OpenInit::try_from(attributes.clone()).unwrap();
         abci_events.push(AbciEvent::from(open_init.clone()));
-        let open_try = OpenTry::from(attributes.clone());
+        let open_try = OpenTry::try_from(attributes.clone()).unwrap();
         abci_events.push(AbciEvent::from(open_try.clone()));
-        let open_ack = OpenAck::from(attributes.clone());
+        let open_ack = OpenAck::try_from(attributes.clone()).unwrap();
         abci_events.push(AbciEvent::from(open_ack.clone()));
-        let open_confirm = OpenConfirm::from(attributes.clone());
+        let open_confirm = OpenConfirm::try_from(attributes.clone()).unwrap();
         abci_events.push(AbciEvent::from(open_confirm.clone()));
-        let close_init = CloseInit::from(attributes.clone());
+        let close_init = CloseInit::try_from(attributes.clone()).unwrap();
         abci_events.push(AbciEvent::from(close_init.clone()));
-        let close_confirm = CloseConfirm::from(attributes);
+        let close_confirm = CloseConfirm::try_from(attributes).unwrap();
         abci_events.push(AbciEvent::from(close_confirm.clone()));
 
         for event in abci_events {
             match try_from_tx(&event) {
                 Some(e) => match e {
-                    IbcEvent::OpenInitChannel(e) => assert_eq!(e.0, open_init.0),
-                    IbcEvent::OpenTryChannel(e) => assert_eq!(e.0, open_try.0),
-                    IbcEvent::OpenAckChannel(e) => assert_eq!(e.0, open_ack.0),
-                    IbcEvent::OpenConfirmChannel(e) => assert_eq!(e.0, open_confirm.0),
-                    IbcEvent::CloseInitChannel(e) => assert_eq!(e.0, close_init.0),
-                    IbcEvent::CloseConfirmChannel(e) => assert_eq!(e.0, close_confirm.0),
+                    IbcEvent::OpenInitChannel(e) => {
+                        assert_eq!(Attributes::from(e), open_init.clone().into())
+                    }
+                    IbcEvent::OpenTryChannel(e) => {
+                        assert_eq!(Attributes::from(e), open_try.clone().into())
+                    }
+                    IbcEvent::OpenAckChannel(e) => {
+                        assert_eq!(Attributes::from(e), open_ack.clone().into())
+                    }
+                    IbcEvent::OpenConfirmChannel(e) => {
+                        assert_eq!(Attributes::from(e), open_confirm.clone().into())
+                    }
+                    IbcEvent::CloseInitChannel(e) => {
+                        assert_eq!(Attributes::from(e), close_init.clone().into())
+                    }
+                    IbcEvent::CloseConfirmChannel(e) => {
+                        assert_eq!(Attributes::from(e), close_confirm.clone().into())
+                    }
                     _ => panic!("unexpected event type"),
                 },
                 None => panic!("converted event was wrong"),
@@ -1040,9 +1164,9 @@ mod tests {
         let packet = Packet {
             sequence: Sequence::from(10),
             source_port: "a_test_port".parse().unwrap(),
-            source_channel: "a_test_channel".parse().unwrap(),
+            source_channel: "channel-0".parse().unwrap(),
             destination_port: "b_test_port".parse().unwrap(),
-            destination_channel: "b_test_channel".parse().unwrap(),
+            destination_channel: "channel-1".parse().unwrap(),
             data: "test_data".as_bytes().to_vec(),
             timeout_height: Height::new(1, 10),
             timeout_timestamp: Timestamp::now(),
