@@ -13,29 +13,29 @@ use crate::prelude::*;
 
 pub(crate) fn process(
     ctx: &dyn ChannelReader,
-    msg: MsgChannelOpenInit,
+    msg: &MsgChannelOpenInit,
 ) -> HandlerResult<ChannelResult, Error> {
     let mut output = HandlerOutput::builder();
 
     // Channel capabilities
-    let channel_cap = ctx.authenticated_capability(&msg.port_id().clone())?;
+    let channel_cap = ctx.authenticated_capability(&msg.port_id)?;
 
-    if msg.channel().connection_hops().len() != 1 {
+    if msg.channel.connection_hops().len() != 1 {
         return Err(Error::invalid_connection_hops_length(
             1,
-            msg.channel().connection_hops().len(),
+            msg.channel.connection_hops().len(),
         ));
     }
 
     // An IBC connection running on the local (host) chain should exist.
-    let conn = ctx.connection_end(&msg.channel().connection_hops()[0])?;
+    let conn = ctx.connection_end(&msg.channel.connection_hops()[0])?;
     let get_versions = conn.versions();
-    let version = match get_versions.as_slice() {
+    let version = match get_versions {
         [version] => version,
         _ => return Err(Error::invalid_version_length_connection()),
     };
 
-    let channel_feature = msg.channel().ordering().to_string();
+    let channel_feature = msg.channel.ordering().to_string();
     if !version.is_supported_feature(channel_feature) {
         return Err(Error::channel_feature_not_suported_by_connection());
     }
@@ -51,17 +51,17 @@ pub(crate) fn process(
 
     let new_channel_end = ChannelEnd::new(
         State::Init,
-        *msg.channel().ordering(),
-        msg.channel().counterparty().clone(),
-        msg.channel().connection_hops().clone(),
-        msg.channel().version().clone(),
+        *msg.channel.ordering(),
+        msg.channel.counterparty().clone(),
+        msg.channel.connection_hops().clone(),
+        msg.channel.version().clone(),
     );
 
     output.log("success: no channel found");
 
     let result = ChannelResult {
-        port_id: msg.port_id().clone(),
-        channel_id: chan_id.clone(),
+        port_id: msg.port_id.clone(),
+        channel_id: chan_id,
         channel_end: new_channel_end,
         channel_id_state: ChannelIdState::Generated,
         channel_cap,
@@ -69,9 +69,14 @@ pub(crate) fn process(
 
     let event_attributes = Attributes {
         channel_id: Some(chan_id),
+        height: ctx.host_height(),
         ..Default::default()
     };
-    output.emit(IbcEvent::OpenInitChannel(event_attributes.into()));
+    output.emit(IbcEvent::OpenInitChannel(
+        event_attributes
+            .try_into()
+            .map_err(|_| Error::missing_channel_id())?,
+    ));
 
     Ok(output.with_result(result))
 }
@@ -88,7 +93,8 @@ mod tests {
     use crate::core::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
     use crate::core::ics03_connection::version::get_compatible_versions;
     use crate::core::ics04_channel::channel::State;
-    use crate::core::ics04_channel::handler::{channel_dispatch, ChannelResult};
+    use crate::core::ics04_channel::context::ChannelReader;
+    use crate::core::ics04_channel::handler::channel_dispatch;
     use crate::core::ics04_channel::msgs::chan_open_init::test_util::get_dummy_raw_msg_chan_open_init;
     use crate::core::ics04_channel::msgs::chan_open_init::MsgChannelOpenInit;
     use crate::core::ics04_channel::msgs::ChannelMsg;
@@ -115,8 +121,8 @@ mod tests {
 
         let init_conn_end = ConnectionEnd::new(
             ConnectionState::Init,
-            msg_conn_init.client_id().clone(),
-            msg_conn_init.counterparty().clone(),
+            msg_conn_init.client_id.clone(),
+            msg_conn_init.counterparty.clone(),
             get_compatible_versions(),
             msg_conn_init.delay_period,
         );
@@ -146,8 +152,7 @@ mod tests {
                     .with_port_capability(
                         MsgChannelOpenInit::try_from(get_dummy_raw_msg_chan_open_init())
                             .unwrap()
-                            .port_id()
-                            .clone(),
+                            .port_id,
                     ),
                 msg: ChannelMsg::ChannelOpenInit(msg_chan_init),
                 want_pass: true,
@@ -157,31 +162,32 @@ mod tests {
         .collect();
 
         for test in tests {
-            let res = channel_dispatch(&test.ctx, test.msg.clone());
+            let res = channel_dispatch(&test.ctx, &test.msg);
             // Additionally check the events and the output objects in the result.
             match res {
-                Ok(proto_output) => {
+                Ok((proto_output, res)) => {
                     assert!(
                         test.want_pass,
                         "chan_open_init: test passed but was supposed to fail for test: {}, \nparams {:?} {:?}",
                         test.name,
-                        test.msg.clone(),
+                        test.msg,
                         test.ctx.clone()
                     );
 
+                    let proto_output = proto_output.with_result(());
                     assert!(!proto_output.events.is_empty()); // Some events must exist.
 
                     // The object in the output is a ChannelEnd, should have init state.
-                    let res: ChannelResult = proto_output.result;
                     assert_eq!(res.channel_end.state().clone(), State::Init);
-                    let msg_init = test.msg.clone();
+                    let msg_init = test.msg;
 
                     if let ChannelMsg::ChannelOpenInit(msg_init) = msg_init {
-                        assert_eq!(res.port_id.clone(), msg_init.port_id().clone());
+                        assert_eq!(res.port_id.clone(), msg_init.port_id.clone());
                     }
 
                     for e in proto_output.events.iter() {
                         assert!(matches!(e, &IbcEvent::OpenInitChannel(_)));
+                        assert_eq!(e.height(), test.ctx.host_height());
                     }
                 }
                 Err(e) => {
