@@ -29,14 +29,6 @@ impl MsgChannelCloseInit {
             signer,
         }
     }
-
-    /// Getter: borrow the `port_id` from this message.
-    pub fn port_id(&self) -> &PortId {
-        &self.port_id
-    }
-    pub fn channel_id(&self) -> &ChannelId {
-        &self.channel_id
-    }
 }
 
 impl Msg for MsgChannelCloseInit {
@@ -148,7 +140,7 @@ mod tests {
             Test {
                 name: "Correct channel identifier".to_string(),
                 raw: RawMsgChannelCloseInit {
-                    channel_id: "channelid34".to_string(),
+                    channel_id: "channel-34".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: true,
@@ -164,7 +156,7 @@ mod tests {
             Test {
                 name: "Bad channel, name too long".to_string(),
                 raw: RawMsgChannelCloseInit {
-                    channel_id: "abcdeasdfasdfasdfasdfasdfasdfasdfasdfdgasdfasdfasdfghijklmnopqrstu".to_string(),
+                    channel_id: "channel-12839128379182739812739879".to_string(),
                     ..default_raw_msg
                 },
                 want_pass: false,
