@@ -46,24 +46,8 @@ impl MsgChannelOpenTry {
             signer,
         }
     }
-
-    /// Getter: borrow the `port_id` from this message.
-    pub fn port_id(&self) -> &PortId {
-        &self.port_id
-    }
-    pub fn previous_channel_id(&self) -> &Option<ChannelId> {
-        &self.previous_channel_id
-    }
-    pub fn counterparty_version(&self) -> &Version {
-        &self.counterparty_version
-    }
-    pub fn channel(&self) -> &ChannelEnd {
-        &self.channel
-    }
-    pub fn proofs(&self) -> &Proofs {
-        &self.proofs
-    }
 }
+
 impl Msg for MsgChannelOpenTry {
     type ValidationError = ChannelError;
     type Raw = RawMsgChannelOpenTry;
@@ -77,7 +61,7 @@ impl Msg for MsgChannelOpenTry {
     }
 
     fn validate_basic(&self) -> Result<(), ValidationError> {
-        match self.channel().counterparty().channel_id() {
+        match self.channel.counterparty().channel_id() {
             None => Err(ValidationError::invalid_counterparty_channel_id()),
             Some(_c) => Ok(()),
         }
@@ -136,9 +120,9 @@ impl From<MsgChannelOpenTry> for RawMsgChannelOpenTry {
             port_id: domain_msg.port_id.to_string(),
             previous_channel_id: domain_msg
                 .previous_channel_id
-                .map_or_else(|| "".to_string(), |v| v.as_str().to_string()),
+                .map_or_else(|| "".to_string(), |v| v.to_string()),
             channel: Some(domain_msg.channel.into()),
-            counterparty_version: domain_msg.counterparty_version.into(),
+            counterparty_version: domain_msg.counterparty_version.to_string(),
             proof_init: domain_msg.proofs.object_proof().clone().into(),
             proof_height: Some(domain_msg.proofs.height().into()),
             signer: domain_msg.signer.to_string(),
@@ -227,7 +211,7 @@ mod tests {
             Test {
                 name: "Correct channel identifier".to_string(),
                 raw: RawMsgChannelOpenTry {
-                    previous_channel_id: "channelid34".to_string(),
+                    previous_channel_id: "channel-34".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: true,
@@ -243,7 +227,7 @@ mod tests {
             Test {
                 name: "Bad channel, name too long".to_string(),
                 raw: RawMsgChannelOpenTry {
-                    previous_channel_id: "abcdefghijkasdfasdfasdfasgdasdgasdfasdfadflmnoasdasdasdfasdfasdfasdfadadgadgadsfpqrstu".to_string(),
+                    previous_channel_id: "channel-12839128379182739812739879".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: false,
