@@ -97,7 +97,7 @@ impl core::fmt::Display for Sequence {
     }
 }
 
-#[derive(Clone, Debug, Default, Hash, PartialEq, Eq, Deserialize, Serialize)]
+#[derive(Clone, Default, Hash, PartialEq, Eq, Deserialize, Serialize)]
 pub struct Packet {
     pub sequence: Sequence,
     pub source_port: PortId,
@@ -110,6 +110,46 @@ pub struct Packet {
     pub timeout_timestamp: Timestamp,
 }
 
+struct PacketData<'a>(&'a [u8]);
+
+impl<'a> core::fmt::Debug for PacketData<'a> {
+    fn fmt(&self, formatter: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
+        write!(formatter, "{:?}", self.0)
+    }
+}
+
+impl core::fmt::Debug for Packet {
+    fn fmt(&self, formatter: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
+        // Remember: if you alter the definition of `Packet`,
+        // 1. update the formatter debug struct builder calls (return object of
+        //    this function)
+        // 2. update this destructuring assignment accordingly
+        let Packet {
+            sequence: _,
+            source_port: _,
+            source_channel: _,
+            destination_port: _,
+            destination_channel: _,
+            data,
+            timeout_height: _,
+            timeout_timestamp: _,
+        } = self;
+        let data_wrapper = PacketData(data);
+
+        formatter
+            .debug_struct("Packet")
+            .field("sequence", &self.sequence)
+            .field("source_port", &self.source_port)
+            .field("source_channel", &self.source_channel)
+            .field("destination_port", &self.destination_port)
+            .field("destination_channel", &self.destination_channel)
+            .field("data", &data_wrapper)
+            .field("timeout_height", &self.timeout_height)
+            .field("timeout_timestamp", &self.timeout_timestamp)
+            .finish()
+    }
+}
+
 impl Packet {
     /// Checks whether a packet from a
     /// [`SendPacket`](crate::core::ics04_channel::events::SendPacket)
@@ -348,7 +388,7 @@ mod tests {
             Test {
                 name: "Src channel validation: correct".to_string(),
                 raw: RawPacket {
-                    source_channel: "srcchannelp34".to_string(),
+                    source_channel: "channel-1".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: true,
@@ -364,7 +404,7 @@ mod tests {
             Test {
                 name: "Bad src channel, name too long".to_string(),
                 raw: RawPacket {
-                    source_channel: "abcdefghijasdfasdfasdfasdfasdfasdfasdfasdfasdfasdfadgasgasdfasdfasdfasdfaklmnopqrstu".to_string(),
+                    source_channel: "channel-12839128379182739812739879".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: false,
@@ -372,7 +412,7 @@ mod tests {
             Test {
                 name: "Dst channel validation: correct".to_string(),
                 raw: RawPacket {
-                    destination_channel: "dstchannelp34".to_string(),
+                    destination_channel: "channel-34".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: true,
@@ -388,7 +428,7 @@ mod tests {
             Test {
                 name: "Bad dst channel, name too long".to_string(),
                 raw: RawPacket {
-                    destination_channel: "abcdefghijasdfasdfasdfasdfasdfasdfasdfasdfasdfasdfadgasgasdfasdfasdfasdfaklmnopqrstu".to_string(),
+                    destination_channel: "channel-12839128379182739812739879".to_string(),
                     ..default_raw_msg.clone()
                 },
                 want_pass: false,
