@@ -1,7 +1,10 @@
-use crate::prelude::*;
-use flex_error::define_error;
+use core::num::ParseIntError;
+
+use flex_error::{define_error, TraceError};
 use serde::Serialize;
 
+use crate::prelude::*;
+
 define_error! {
     #[derive(Debug, PartialEq, Eq, Serialize)]
     ValidationError {
@@ -29,6 +32,13 @@ define_error! {
             { id: String }
             | e | { format_args!("chain identifiers are expected to be in epoch format {0}", e.id) },
 
+        ChannelIdInvalidFormat
+            | _ | { "channel identifiers are expected to be in `channel-{N}` format" },
+
+        ChannelIdParseFailure
+            [ TraceError<ParseIntError> ]
+            | _ | { "failed to parse channel identifier" },
+
         InvalidCounterpartyChannelId
             |_| { "Invalid channel id in counterparty" }
     }
