@@ -3,13 +3,16 @@
 use crate::prelude::*;
 
 use alloc::collections::btree_map::BTreeMap;
+use alloc::sync::Arc;
+use core::borrow::Borrow;
 use core::cmp::min;
+use core::fmt::Debug;
+use core::ops::{Add, Sub};
+use core::time::Duration;
 
-use tracing::debug;
-
-use prost_types::Any;
+use ibc_proto::google::protobuf::Any;
 use sha2::Digest;
-use tendermint::Time;
+use tracing::debug;
 
 use crate::applications::ics20_fungible_token_transfer::context::Ics20Context;
 use crate::clients::ics07_tendermint::client_state::test_util::get_dummy_tendermint_client_state;
@@ -26,12 +29,15 @@ use crate::core::ics04_channel::channel::ChannelEnd;
 use crate::core::ics04_channel::context::{ChannelKeeper, ChannelReader};
 use crate::core::ics04_channel::error::Error as Ics04Error;
 use crate::core::ics04_channel::packet::{Receipt, Sequence};
-use crate::core::ics05_port::capabilities::Capability;
-use crate::core::ics05_port::context::PortReader;
+use crate::core::ics05_port::capabilities::{
+    Capability, CapabilityName, ChannelCapability, PortCapability,
+};
+use crate::core::ics05_port::context::{CapabilityReader, PortReader};
 use crate::core::ics05_port::error::Error as Ics05Error;
+use crate::core::ics05_port::error::Error;
 use crate::core::ics23_commitment::commitment::CommitmentPrefix;
 use crate::core::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
-use crate::core::ics26_routing::context::Ics26Context;
+use crate::core::ics26_routing::context::{Ics26Context, Module, ModuleId, Router, RouterBuilder};
 use crate::core::ics26_routing::handler::{deliver, dispatch};
 use crate::core::ics26_routing::msgs::Ics26Envelope;
 use crate::events::IbcEvent;
@@ -43,7 +49,8 @@ use crate::relayer::ics18_relayer::error::Error as Ics18Error;
 use crate::signer::Signer;
 use crate::timestamp::Timestamp;
 use crate::Height;
-use core::time::Duration;
+
+pub const DEFAULT_BLOCK_TIME_SECS: u64 = 3;
 
 /// A context implementing the dependencies necessary for testing any IBC module.
 #[derive(Clone, Debug)]
@@ -57,12 +64,6 @@ pub struct MockContext {
     /// Maximum size for the history of the host chain. Any block older than this is pruned.
     max_history_size: usize,
 
-    /// Highest height (i.e., most recent) of the blocks in the history.
-    latest_height: Height,
-
-    /// Highest timestamp, i.e., of the most recent block in the history.
-    timestamp: Timestamp,
-
     /// The chain of blocks underlying this context. A vector of size up to `max_history_size`
     /// blocks, ascending order by their height (latest block is on the last position).
     history: Vec<HostBlock>,
@@ -110,13 +111,19 @@ pub struct MockContext {
     packet_acknowledgement: BTreeMap<(PortId, ChannelId, Sequence), String>,
 
     /// Maps ports to their capabilities
-    port_capabilities: BTreeMap<PortId, Capability>,
+    port_capabilities: BTreeMap<PortId, (ModuleId, PortCapability)>,
 
     /// Constant-size commitments to packets data fields
     packet_commitment: BTreeMap<(PortId, ChannelId, Sequence), String>,
 
     // Used by unordered channel
     packet_receipt: BTreeMap<(PortId, ChannelId, Sequence), Receipt>,
+
+    /// Average time duration between blocks
+    block_time: Duration,
+
+    /// ICS26 router impl
+    router: MockRouter,
 }
 
 /// Returns a MockContext with bare minimum initialization: no clients, no connections and no channels are
@@ -151,7 +158,12 @@ impl MockContext {
             "The chain must have a non-zero max_history_size"
         );
 
-        // Compute the number of blocks to store. If latest_height is 0, nothing is stored.
+        assert_ne!(
+            latest_height.revision_height, 0,
+            "The chain must have a non-zero revision_height"
+        );
+
+        // Compute the number of blocks to store.
         let n = min(max_history_size as u64, latest_height.revision_height);
 
         assert_eq!(
@@ -160,19 +172,24 @@ impl MockContext {
             "The version in the chain identifier must match the version in the latest height"
         );
 
+        let block_time = Duration::from_secs(DEFAULT_BLOCK_TIME_SECS);
+        let next_block_timestamp = Timestamp::now().add(block_time).unwrap();
         MockContext {
             host_chain_type: host_type,
             host_chain_id: host_id.clone(),
             max_history_size,
-            latest_height,
-            timestamp: Default::default(),
             history: (0..n)
                 .rev()
                 .map(|i| {
+                    // generate blocks with timestamps -> N, N - BT, N - 2BT, ...
+                    // where N = now(), BT = block_time
                     HostBlock::generate_block(
                         host_id.clone(),
                         host_type,
                         latest_height.sub(i).unwrap().revision_height,
+                        next_block_timestamp
+                            .sub(Duration::from_secs(DEFAULT_BLOCK_TIME_SECS * (i + 1)))
+                            .unwrap(),
                     )
                 })
                 .collect(),
@@ -193,6 +210,8 @@ impl MockContext {
             packet_acknowledgement: Default::default(),
             connection_ids_counter: 0,
             channel_ids_counter: 0,
+            block_time,
+            router: Default::default(),
         }
     }
 
@@ -230,6 +249,7 @@ impl MockContext {
                 let light_block = HostBlock::generate_tm_block(
                     self.host_chain_id.clone(),
                     cs_height.revision_height,
+                    Timestamp::now(),
                 );
 
                 let consensus_state = AnyConsensusState::from(light_block.clone());
@@ -264,6 +284,7 @@ impl MockContext {
         let prev_cs_height = cs_height.clone().sub(1).unwrap_or(client_state_height);
 
         let client_type = client_type.unwrap_or(ClientType::Mock);
+        let now = Timestamp::now();
 
         let (client_state, consensus_state) = match client_type {
             // If it's a mock client, create the corresponding mock states.
@@ -276,6 +297,7 @@ impl MockContext {
                 let light_block = HostBlock::generate_tm_block(
                     self.host_chain_id.clone(),
                     cs_height.revision_height,
+                    now,
                 );
 
                 let consensus_state = AnyConsensusState::from(light_block.clone());
@@ -295,6 +317,7 @@ impl MockContext {
                 let light_block = HostBlock::generate_tm_block(
                     self.host_chain_id.clone(),
                     prev_cs_height.revision_height,
+                    now.sub(self.block_time).unwrap(),
                 );
                 AnyConsensusState::from(light_block)
             }
@@ -330,7 +353,7 @@ impl MockContext {
     }
 
     pub fn with_port_capability(mut self, port_id: PortId) -> Self {
-        self.port_capabilities.insert(port_id, Capability::new());
+        self.add_port(port_id);
         self
     }
 
@@ -388,21 +411,18 @@ impl MockContext {
         }
     }
 
-    pub fn with_timestamp(self, timestamp: Timestamp) -> Self {
-        Self { timestamp, ..self }
-    }
-
     pub fn with_height(self, target_height: Height) -> Self {
-        if target_height.revision_number > self.latest_height.revision_number {
+        let latest_height = self.latest_height();
+        if target_height.revision_number > latest_height.revision_number {
             unimplemented!()
-        } else if target_height.revision_number < self.latest_height.revision_number {
+        } else if target_height.revision_number < latest_height.revision_number {
             panic!("Cannot rewind history of the chain to a smaller revision number!")
-        } else if target_height.revision_height < self.latest_height.revision_height {
+        } else if target_height.revision_height < latest_height.revision_height {
             panic!("Cannot rewind history of the chain to a smaller revision height!")
-        } else if target_height.revision_height > self.latest_height.revision_height {
+        } else if target_height.revision_height > latest_height.revision_height {
             // Repeatedly advance the host chain height till we hit the desired height
             let mut ctx = MockContext { ..self };
-            while ctx.latest_height.revision_height < target_height.revision_height {
+            while ctx.latest_height().revision_height < target_height.revision_height {
                 ctx.advance_host_chain_height()
             }
             ctx
@@ -427,11 +447,15 @@ impl MockContext {
         }
     }
 
+    pub fn with_router(self, router: MockRouter) -> Self {
+        Self { router, ..self }
+    }
+
     /// Accessor for a block of the local (host) chain from this context.
     /// Returns `None` if the block at the requested height does not exist.
     pub fn host_block(&self, target_height: Height) -> Option<&HostBlock> {
         let target = target_height.revision_height as usize;
-        let latest = self.latest_height.revision_height as usize;
+        let latest = self.latest_height().revision_height as usize;
 
         // Check that the block is not too advanced, nor has it been pruned.
         if (target > latest) || (target <= latest - self.history.len()) {
@@ -443,10 +467,12 @@ impl MockContext {
 
     /// Triggers the advancing of the host chain, by extending the history of blocks (or headers).
     pub fn advance_host_chain_height(&mut self) {
+        let latest_block = self.history.last().expect("history cannot be empty");
         let new_block = HostBlock::generate_block(
             self.host_chain_id.clone(),
             self.host_chain_type,
-            self.latest_height.increment().revision_height,
+            latest_block.height().increment().revision_height,
+            latest_block.timestamp().add(self.block_time).unwrap(),
         );
 
         // Append the new header at the tip of the history.
@@ -458,14 +484,13 @@ impl MockContext {
             // History is not full yet.
             self.history.push(new_block);
         }
-        self.latest_height = self.latest_height.increment();
     }
 
     /// A datagram passes from the relayer to the IBC module (on host chain).
     /// Alternative method to `Ics18Context::send` that does not exercise any serialization.
     /// Used in testing the Ics18 algorithms, hence this may return a Ics18Error.
     pub fn deliver(&mut self, msg: Ics26Envelope) -> Result<(), Ics18Error> {
-        dispatch(Time::now(), self, msg).map_err(Ics18Error::transaction_failed)?;
+        dispatch(self, msg).map_err(Ics18Error::transaction_failed)?;
         // Create a new block.
         self.advance_host_chain_height();
         Ok(())
@@ -483,7 +508,7 @@ impl MockContext {
             // Get the highest block.
             let lh = &self.history[self.history.len() - 1];
             // Check latest is properly updated with highest header height.
-            if lh.height() != self.latest_height {
+            if lh.height() != self.latest_height() {
                 return Err("latest height is not updated".to_string());
             }
         }
@@ -500,7 +525,14 @@ impl MockContext {
     }
 
     pub fn add_port(&mut self, port_id: PortId) {
-        self.port_capabilities.insert(port_id, Capability::new());
+        let module_id = ModuleId::new(format!("module{}", port_id).into()).unwrap();
+        self.port_capabilities
+            .insert(port_id, (module_id, Capability::new().into()));
+    }
+
+    pub fn scope_port_to_module(&mut self, port_id: PortId, module_id: ModuleId) {
+        self.port_capabilities
+            .insert(port_id, (module_id, Capability::new().into()));
     }
 
     pub fn consensus_states(&self, client_id: &ClientId) -> Vec<AnyConsensusStateWithHeight> {
@@ -528,22 +560,81 @@ impl MockContext {
             .get(height)
             .unwrap()
     }
+
+    #[inline]
+    fn latest_height(&self) -> Height {
+        self.history
+            .last()
+            .expect("history cannot be empty")
+            .height()
+    }
 }
 
-impl Ics26Context for MockContext {}
+#[derive(Default)]
+pub struct MockRouterBuilder(MockRouter);
 
-impl Ics20Context for MockContext {}
+impl RouterBuilder for MockRouterBuilder {
+    type Router = MockRouter;
 
-impl PortReader for MockContext {
-    fn lookup_module_by_port(&self, port_id: &PortId) -> Result<Capability, Ics05Error> {
-        match self.port_capabilities.get(port_id) {
-            Some(cap) => Ok(cap.clone()),
-            None => Err(Ics05Error::unknown_port(port_id.clone())),
+    fn add_route(mut self, module_id: ModuleId, module: impl Module) -> Result<Self, String> {
+        match self.0 .0.insert(module_id, Arc::new(module)) {
+            None => Ok(self),
+            Some(_) => Err("Duplicate module_id".to_owned()),
         }
     }
 
-    fn authenticate(&self, _cap: &Capability, _port_id: &PortId) -> bool {
-        true
+    fn build(self) -> Self::Router {
+        self.0
+    }
+}
+
+#[derive(Clone, Debug, Default)]
+pub struct MockRouter(BTreeMap<ModuleId, Arc<dyn Module>>);
+
+impl Router for MockRouter {
+    fn get_route_mut(&mut self, module_id: &impl Borrow<ModuleId>) -> Option<&mut dyn Module> {
+        self.0.get_mut(module_id.borrow()).and_then(Arc::get_mut)
+    }
+
+    fn has_route(&self, module_id: &impl Borrow<ModuleId>) -> bool {
+        self.0.get(module_id.borrow()).is_some()
+    }
+}
+
+impl Ics26Context for MockContext {
+    type Router = MockRouter;
+
+    fn router(&self) -> &Self::Router {
+        &self.router
+    }
+
+    fn router_mut(&mut self) -> &mut Self::Router {
+        &mut self.router
+    }
+}
+
+impl Ics20Context for MockContext {}
+
+impl CapabilityReader for MockContext {
+    fn get_capability(&self, _name: &CapabilityName) -> Result<Capability, Ics05Error> {
+        todo!()
+    }
+
+    fn authenticate_capability(
+        &self,
+        _name: &CapabilityName,
+        _capability: &Capability,
+    ) -> Result<(), Ics05Error> {
+        Ok(())
+    }
+}
+
+impl PortReader for MockContext {
+    fn lookup_module_by_port(&self, port_id: &PortId) -> Result<(ModuleId, PortCapability), Error> {
+        match self.port_capabilities.get(port_id) {
+            Some((mod_id, mod_cap)) => Ok((mod_id.clone(), mod_cap.clone())),
+            None => Err(Ics05Error::unknown_port(port_id.clone())),
+        }
     }
 }
 
@@ -551,10 +642,7 @@ impl ChannelReader for MockContext {
     fn channel_end(&self, pcid: &(PortId, ChannelId)) -> Result<ChannelEnd, Ics04Error> {
         match self.channels.get(pcid) {
             Some(channel_end) => Ok(channel_end.clone()),
-            None => Err(Ics04Error::channel_not_found(
-                pcid.0.clone(),
-                pcid.1.clone(),
-            )),
+            None => Err(Ics04Error::channel_not_found(pcid.0.clone(), pcid.1)),
         }
     }
 
@@ -586,13 +674,13 @@ impl ChannelReader for MockContext {
             .map_err(|e| Ics04Error::ics03_connection(Ics03Error::ics02_client(e)))
     }
 
-    fn authenticated_capability(&self, port_id: &PortId) -> Result<Capability, Ics04Error> {
+    fn authenticated_capability(&self, port_id: &PortId) -> Result<ChannelCapability, Ics04Error> {
         match PortReader::lookup_module_by_port(self, port_id) {
-            Ok(key) => {
-                if !PortReader::authenticate(self, &key, port_id) {
+            Ok((_, key)) => {
+                if !PortReader::authenticate(self, port_id.clone(), &key) {
                     Err(Ics04Error::invalid_port_capability())
                 } else {
-                    Ok(key)
+                    Ok(Capability::from(key).into())
                 }
             }
             Err(e) if e.detail() == Ics05Error::unknown_port(port_id.clone()).detail() => {
@@ -668,11 +756,20 @@ impl ChannelReader for MockContext {
     }
 
     fn host_height(&self) -> Height {
-        self.latest_height
+        self.latest_height()
     }
 
     fn host_timestamp(&self) -> Timestamp {
-        self.timestamp
+        ClientReader::host_timestamp(self)
+    }
+
+    fn host_consensus_state(&self, height: Height) -> Result<AnyConsensusState, Ics04Error> {
+        ConnectionReader::host_consensus_state(self, height).map_err(Ics04Error::ics03_connection)
+    }
+
+    fn pending_host_consensus_state(&self) -> Result<AnyConsensusState, Ics04Error> {
+        ClientReader::pending_host_consensus_state(self)
+            .map_err(|e| Ics04Error::ics03_connection(Ics03Error::ics02_client(e)))
     }
 
     fn client_update_time(
@@ -714,7 +811,17 @@ impl ChannelReader for MockContext {
     }
 
     fn max_expected_time_per_block(&self) -> Duration {
-        Duration::from_secs(10)
+        self.block_time
+    }
+
+    fn lookup_module_by_channel(
+        &self,
+        _channel_id: &ChannelId,
+        port_id: &PortId,
+    ) -> Result<(ModuleId, ChannelCapability), Ics04Error> {
+        self.lookup_module_by_port(port_id)
+            .map(|(mid, pcap)| (mid, ChannelCapability::from(Capability::from(pcap))))
+            .map_err(Ics04Error::ics05_port)
     }
 }
 
@@ -835,7 +942,7 @@ impl ConnectionReader for MockContext {
     }
 
     fn host_current_height(&self) -> Height {
-        self.latest_height
+        self.latest_height()
     }
 
     fn host_oldest_height(&self) -> Height {
@@ -858,10 +965,7 @@ impl ConnectionReader for MockContext {
     }
 
     fn host_consensus_state(&self, height: Height) -> Result<AnyConsensusState, Ics03Error> {
-        match self.host_block(height) {
-            Some(block_ref) => Ok(block_ref.clone().into()),
-            None => Err(Ics03Error::missing_local_consensus_state(height)),
-        }
+        ClientReader::host_consensus_state(self, height).map_err(Ics03Error::ics02_client)
     }
 
     fn connection_counter(&self) -> Result<u64, Ics03Error> {
@@ -988,7 +1092,27 @@ impl ClientReader for MockContext {
     }
 
     fn host_height(&self) -> Height {
-        self.latest_height
+        self.latest_height()
+    }
+
+    fn host_timestamp(&self) -> Timestamp {
+        self.history
+            .last()
+            .expect("history cannot be empty")
+            .timestamp()
+            .add(self.block_time)
+            .unwrap()
+    }
+
+    fn host_consensus_state(&self, height: Height) -> Result<AnyConsensusState, Ics02Error> {
+        match self.host_block(height) {
+            Some(block_ref) => Ok(block_ref.clone().into()),
+            None => Err(Ics02Error::missing_local_consensus_state(height)),
+        }
+    }
+
+    fn pending_host_consensus_state(&self) -> Result<AnyConsensusState, Ics02Error> {
+        Err(Ics02Error::missing_local_consensus_state(Height::zero()))
     }
 
     fn client_counter(&self) -> Result<u64, Ics02Error> {
@@ -1049,10 +1173,15 @@ impl ClientKeeper for MockContext {
         self.client_ids_counter += 1
     }
 
-    fn store_update_time(&mut self, client_id: ClientId, height: Height) -> Result<(), Ics02Error> {
+    fn store_update_time(
+        &mut self,
+        client_id: ClientId,
+        height: Height,
+        timestamp: Timestamp,
+    ) -> Result<(), Ics02Error> {
         let _ = self
             .client_processed_times
-            .insert((client_id, height), ChannelReader::host_timestamp(self));
+            .insert((client_id, height), timestamp);
         Ok(())
     }
 
@@ -1060,10 +1189,11 @@ impl ClientKeeper for MockContext {
         &mut self,
         client_id: ClientId,
         height: Height,
+        host_height: Height,
     ) -> Result<(), Ics02Error> {
         let _ = self
             .client_processed_heights
-            .insert((client_id, height), ClientReader::host_height(self));
+            .insert((client_id, height), host_height);
         Ok(())
     }
 }
@@ -1085,10 +1215,13 @@ impl Ics18Context for MockContext {
 
     fn send(&mut self, msgs: Vec<Any>) -> Result<Vec<IbcEvent>, Ics18Error> {
         // Forward call to Ics26 delivery method.
-        let events = deliver(Time::now(), self, msgs).map_err(Ics18Error::transaction_failed)?;
-
+        let mut all_events = vec![];
+        for msg in msgs {
+            let (mut events, _) = deliver(self, msg).map_err(Ics18Error::transaction_failed)?;
+            all_events.append(&mut events);
+        }
         self.advance_host_chain_height(); // Advance chain height
-        Ok(events)
+        Ok(all_events)
     }
 
     fn signer(&self) -> Signer {
@@ -1098,12 +1231,26 @@ impl Ics18Context for MockContext {
 
 #[cfg(test)]
 mod tests {
+    use test_log::test;
+
+    use alloc::str::FromStr;
+
+    use crate::core::ics04_channel::channel::{Counterparty, Order};
+    use crate::core::ics04_channel::error::Error;
+    use crate::core::ics04_channel::packet::Packet;
+    use crate::core::ics04_channel::Version;
+    use crate::core::ics05_port::capabilities::ChannelCapability;
     use crate::core::ics24_host::identifier::ChainId;
+    use crate::core::ics24_host::identifier::{ChannelId, ConnectionId, PortId};
+    use crate::core::ics26_routing::context::{
+        Acknowledgement, Module, ModuleId, ModuleOutput, OnRecvPacketAck, Router, RouterBuilder,
+    };
     use crate::mock::context::MockContext;
+    use crate::mock::context::MockRouterBuilder;
     use crate::mock::host::HostType;
     use crate::prelude::*;
+    use crate::signer::Signer;
     use crate::Height;
-    use test_log::test;
 
     #[test]
     fn test_history_manipulation() {
@@ -1119,8 +1266,8 @@ mod tests {
                 ctx: MockContext::new(
                     ChainId::new("mockgaia".to_string(), cv),
                     HostType::Mock,
-                    1,
-                    Height::new(cv, 0),
+                    2,
+                    Height::new(cv, 1),
                 ),
             },
             Test {
@@ -1128,8 +1275,8 @@ mod tests {
                 ctx: MockContext::new(
                     ChainId::new("mocksgaia".to_string(), cv),
                     HostType::SyntheticTendermint,
-                    1,
-                    Height::new(cv, 0),
+                    2,
+                    Height::new(cv, 1),
                 ),
             },
             Test {
@@ -1210,37 +1357,149 @@ mod tests {
             // All tests should yield a valid context after initialization.
             assert!(
                 test.ctx.validate().is_ok(),
-                "Failed in test {} while validating context {:?}",
+                "failed in test {} while validating context {:?}",
                 test.name,
                 test.ctx
             );
 
-            let current_height = test.ctx.latest_height;
+            let current_height = test.ctx.latest_height();
 
             // After advancing the chain's height, the context should still be valid.
             test.ctx.advance_host_chain_height();
             assert!(
                 test.ctx.validate().is_ok(),
-                "Failed in test {} while validating context {:?}",
+                "failed in test {} while validating context {:?}",
                 test.name,
                 test.ctx
             );
 
             let next_height = current_height.increment();
             assert_eq!(
-                test.ctx.latest_height, next_height,
-                "Failed while increasing height for context {:?}",
+                test.ctx.latest_height(),
+                next_height,
+                "failed while increasing height for context {:?}",
                 test.ctx
             );
             if current_height > Height::new(cv, 0) {
                 assert_eq!(
                     test.ctx.host_block(current_height).unwrap().height(),
                     current_height,
-                    "Failed while fetching height {:?} of context {:?}",
+                    "failed while fetching height {:?} of context {:?}",
                     current_height,
                     test.ctx
                 );
             }
         }
     }
+
+    #[test]
+    fn test_router() {
+        #[derive(Default)]
+        struct MockAck(Vec<u8>);
+
+        impl AsRef<[u8]> for MockAck {
+            fn as_ref(&self) -> &[u8] {
+                self.0.as_slice()
+            }
+        }
+
+        impl Acknowledgement for MockAck {}
+
+        #[derive(Debug, Default)]
+        struct FooModule {
+            counter: usize,
+        }
+
+        impl Module for FooModule {
+            fn on_chan_open_try(
+                &mut self,
+                _output: &mut ModuleOutput,
+                _order: Order,
+                _connection_hops: &[ConnectionId],
+                _port_id: &PortId,
+                _channel_id: &ChannelId,
+                _channel_cap: &ChannelCapability,
+                _counterparty: &Counterparty,
+                counterparty_version: &Version,
+            ) -> Result<Version, Error> {
+                Ok(counterparty_version.clone())
+            }
+
+            fn on_recv_packet(
+                &self,
+                _output: &mut ModuleOutput,
+                _packet: &Packet,
+                _relayer: &Signer,
+            ) -> OnRecvPacketAck {
+                OnRecvPacketAck::Successful(
+                    Box::new(MockAck::default()),
+                    Box::new(|module| {
+                        let module = module.downcast_mut::<FooModule>().unwrap();
+                        module.counter += 1;
+                    }),
+                )
+            }
+        }
+
+        #[derive(Debug, Default)]
+        struct BarModule;
+
+        impl Module for BarModule {
+            fn on_chan_open_try(
+                &mut self,
+                _output: &mut ModuleOutput,
+                _order: Order,
+                _connection_hops: &[ConnectionId],
+                _port_id: &PortId,
+                _channel_id: &ChannelId,
+                _channel_cap: &ChannelCapability,
+                _counterparty: &Counterparty,
+                counterparty_version: &Version,
+            ) -> Result<Version, Error> {
+                Ok(counterparty_version.clone())
+            }
+        }
+
+        let r = MockRouterBuilder::default()
+            .add_route("foomodule".parse().unwrap(), FooModule::default())
+            .unwrap()
+            .add_route("barmodule".parse().unwrap(), BarModule::default())
+            .unwrap()
+            .build();
+
+        let mut ctx = MockContext::new(
+            ChainId::new("mockgaia".to_string(), 1),
+            HostType::Mock,
+            1,
+            Height::new(1, 1),
+        )
+        .with_router(r);
+
+        let mut on_recv_packet_result = |module_id: &'static str| {
+            let module_id = ModuleId::from_str(module_id).unwrap();
+            let m = ctx.router.get_route_mut(&module_id).unwrap();
+            let result = m.on_recv_packet(
+                &mut ModuleOutput::builder().with_result(()),
+                &Packet::default(),
+                &Signer::new(""),
+            );
+            (module_id, result)
+        };
+
+        let results = vec![
+            on_recv_packet_result("foomodule"),
+            on_recv_packet_result("barmodule"),
+        ];
+        results
+            .into_iter()
+            .filter_map(|(mid, result)| match result {
+                OnRecvPacketAck::Nil(write_fn) | OnRecvPacketAck::Successful(_, write_fn) => {
+                    Some((mid, write_fn))
+                }
+                _ => None,
+            })
+            .for_each(|(mid, write_fn)| {
+                write_fn(ctx.router.get_route_mut(&mid).unwrap().as_any_mut())
+            });
+    }
 }
